# Copyright 2014 Canonical Ltd.  This software is licensed under the
# GNU General Public License version 3 (see the file LICENSE).
from __future__ import absolute_import
import os
from shutil import rmtree
import subprocess
from tempfile import mkdtemp

import mojo
import mojo.spec
import mojo.workspace


class MojoTestCaseMixin():

    def override_environ(self, name, value=None):
        """Temporarily override an environment variable."""
        old_value = os.environ.pop(name, None)
        if value is not None:
            os.environ[name] = value
        if old_value is not None:
            self.addCleanup(os.environ.__setitem__, name, old_value)
        else:
            self.addCleanup(os.environ.pop, name, None)

    def create_unique_bzr_branch(self, directory, filenames=None,
                                 manifests=None):
        bzr_home = mkdtemp()
        self.addCleanup(rmtree, bzr_home)
        self.override_environ('BZR_HOME', bzr_home)
        self.override_environ('BRZ_HOME', bzr_home)

        try:
            cmd = ['bzr', 'init', directory]
            subprocess.check_call(cmd, stderr=subprocess.STDOUT)
            cmd = ['bzr', 'whoami', '-d', directory, '--branch', 'Mojo Tests <mojo-tests@example.com>']
            subprocess.check_call(cmd, stderr=subprocess.STDOUT)
        except subprocess.CalledProcessError:
            raise

        if filenames is None:
            filenames = ('test_file',)
        for filename in filenames:
            filepath = os.path.join(directory, filename)
            if not os.path.exists(os.path.dirname(filepath)):
                os.makedirs(os.path.dirname(filepath))
            with open(filepath, 'w') as f:
                f.write('deploy-config:\n')
                f.write('  series: {{ series }}\n')
                f.write('  services:\n')
                f.write('    apache2:\n')
                f.write('      charm: {{ charm_dir }}/apache2\n')
                f.flush()
        # Ensure there's a valid manifest as part of the spec.
        with open(os.path.join(directory, 'manifest'), 'w') as f:
            f.write("\n".join(["collect", "repo", "deploy"]))

        manifests = manifests or {'manifest': 'collect\nrepo\ndeploy'}
        for manifest_path, content in manifests.items():
            parent, _ = os.path.split(manifest_path)
            full_parent = os.path.join(directory, parent)
            if not os.path.exists(full_parent):
                os.makedirs(full_parent)
            with open(os.path.join(directory, manifest_path), 'w+') as fp:
                fp.write(content)

        filenames = list(filenames) + list(manifests.keys())

        with mojo.utils.chdir(directory):
            for filename in filenames:
                cmd = ['bzr', 'add', filename]
                subprocess.check_call(cmd, stderr=subprocess.STDOUT)
            cmd = ['bzr', 'commit', '-m', 'commit']
            subprocess.check_call(cmd, stderr=subprocess.STDOUT)

    def create_spec_dir(self, configs=None, stage=None, path=None,
                        manifests=None):
        """Return a tmp dir initialised as a spec branch with configs."""
        if path is None:
            path = mkdtemp()
            self.addCleanup(rmtree, path)
        configs = configs or []
        if stage is not None:
            os.makedirs(os.path.join(path, stage))
        self.create_unique_bzr_branch(path, configs, manifests)
        return path

    def create_spec(self, configs=None, stage=None, path=None,
                    manifests=None):
        spec_dir = self.create_spec_dir(configs=configs, stage=stage,
                                        path=path, manifests=manifests)
        return mojo.spec.Spec(spec_dir)

    def create_workspace(self, name="workspace"):
        workspace_dir = mkdtemp()
        self.addCleanup(rmtree, workspace_dir)
        workspace = mojo.workspace.Workspace(name, workspace_dir)
        workspace.prepare()
        os.mkdir(workspace.repo_dir)
        return workspace


# From http://stackoverflow.com/a/1160227/523729
def touch(filename, times=None):
    with open(filename, 'a'):
        os.utime(filename, times)
